#!/usr/bin/perl

# This is required during the BEGIN block by all the final delivery processes.
# It includes our basic configuration file and several startup routines.
# 
# We pull our config from the fd_settings table in the vpopmail database.
# 
# Japheth Cleaver <cleaver@redwire.net>
# 09/26/2007
# 

use strict;
our $fdVersion = '8.3.7';

require DBI;
require Sys::Hostname;


# Our database handles and other variables used in different places
  our ($DBH, $DBHread, $QBH, $QBHread);
  our %defaultConfigData;
  our (%domainResolve, %domainDirs, %lastQmailFileMtime, $virtUserFileMtime);


# The location of some important configuration files, with which to bootstrap
# our state. Most things are then read from the database, with defaults in here
# 
  my $mysqlConfigFile		= '/etc/vpopmail/vpopmail.mysql';
  my $virtUserFile		= '/etc/qmail/users/assign';
# this file is optional
  my $queueMysqlConfigFile	= '/etc/vpopmail/fd.mysql';

  our %fdConfig = ( 
	tablePrefix		=> 'fd_',
	scratchDir 		=> '/tmp',
	defaultDomainFile	=> '/etc/vpopmail/defaultdomain',
	virusReplacementText	=> '/etc/vpopmail/virus-replacement.msg',
	vrecordLogFile		=> '/var/log/vpopmail/vrecordmail.log',
	realDomainDirLoc	=> '/var/vpopmail/domains',
	queueTableInfix		=> 'queue_',
	recentTableInfix	=> 'recent_',
	storageTableInfix	=> 'mail_',
	storeHeader		=> 1,
	storeCopy		=> 0,
  );

# But... use a ramdisk is one is available
  $fdConfig{'scratchDir'} = '/dev/shm' if -d '/dev/shm' && -w _;


##########################################################################
# Get our configuration data from the fd_settings database
##########################################################################

  my ($settingsReadHandle, undef) = dbhConnect('vpopmail');

  my $getSettings =
	$settingsReadHandle->prepare('SELECT * FROM fd_settings') or die "Can't prepare fd_settings statement: " . DBI::errstr();

  $getSettings->execute() or die "Can't execute fd_settings statement: " . DBI::errstr();

  if ($getSettings->rows()) {
	my @data;
	$fdConfig{$data[0]} = $data[1] while (@data = $getSettings->fetchrow_array);
  } else { 
	warn 'Empty fd_settings table';
  };

  $getSettings->finish();
  $settingsReadHandle->disconnect();

  my @datetime=(localtime)[0,1,2,3,4,5];
  $datetime[4]++; $datetime[5] += 1900;
  our $yearmon = sprintf '%04d%02d', @datetime[5,4];

  our $thisHost	  = $fdConfig{'thisHost'} ||= Sys::Hostname::hostname() or die "Can't get local hostname";
  $ENV{'TMPDIR'}  = our $scratchDir = $fdConfig{'scratchDir'};

  our $queueTable	= join($fdConfig{'queueTableInfix'}, $fdConfig{'tablePrefix'}, $yearmon);
  our $recentTable	= join($fdConfig{'recentTableInfix'},  $fdConfig{'tablePrefix'}, $yearmon);
  our $storageTable	= join($fdConfig{'storageTableInfix'},  $fdConfig{'tablePrefix'}, $yearmon);

  open (FILE, $fdConfig{'defaultDomainFile'}) or die "Can't open $fdConfig{'defaultDomainFile'}: $!";
    chomp (our $defaultDomain = <FILE>);
  close FILE;


  return 1;


#########################################################################
# 	Subroutines							#
#########################################################################

## 
## Connect to a database, given either 'vpopmail' or 'fdqueue', based on
## a vpopmail config file file
## 

sub dbhConnect {

  my ($updateDBH, $readDBH);

  # What are we finding? can be vpopmail or fdqueue or... ?
    my $type = $_[0] || 'vpopmail';


  if ($type eq 'vpopmail' || $type eq 'fdqueue') {
  # Get our file name...
    my $configFile =	$type eq 'vpopmail'	? $mysqlConfigFile :
			$type eq 'fdqueue'	? $queueMysqlConfigFile :
			die "BUG: Unknown database request: $type";

  # Open our file and read out our config
    open (FILE, $configFile) or die "Can't open $configFile: $!";
    while (<FILE>) {

      # Skip comments/blanks
	next if /^\s*#/ || /^\s*$/; chomp;
      # ($server | $port | $dbuser | $dbpass | $dbname)
	my @sqlConfig = split(/\|/, $_, 5);

	my $dsnString = 
	   $sqlConfig[0] eq 'localhost' ?
		join($sqlConfig[4], 'DBI:mysql:database=', ';mysql_client_found_rows=0') :
		join('',  'DBI:mysql:database=', $sqlConfig[4], ';host=', $sqlConfig[0], ';port=', $sqlConfig[1], ';mysql_client_found_rows=0');

      # Read-only version comes first, then the write version (if any)
	unless ($readDBH) {
	    $readDBH = DBI->connect($dsnString, @sqlConfig[2,3], { RaiseError => 0, PrintError => 0 });
	    $readDBH->{'mysql_auto_reconnect'} = 1;
	} else {
	    $updateDBH = DBI->connect($dsnString, @sqlConfig[2,3], { RaiseError => 0, PrintError => 0 });
	    $updateDBH->{'mysql_auto_reconnect'} = 1;
	    last;
        };
    };
    close FILE;

  # Hmm.. if we don't have a write version, then the read version WAS our write version!
    return ($readDBH, $updateDBH || $readDBH);

  } elsif ($type eq 'awl') {

    my %awlData;
    chomp (my @saAWLsettings = `grep -h ^user_awl_ /etc/mail/spamassassin/*`);

    unless (@saAWLsettings) {
	warn "No AWL settings found in /etc/mail/spamassassin/; skipping";
	return (undef, undef);
    };
    
    m/^([^\s]+)\s+([^\s]+)\s*$/ and $awlData{$1}=$2 foreach (@saAWLsettings);

    my $dsnString = $awlData{user_awl_dsn} . ';mysql_client_found_rows=0';

    my  $awlDBH = DBI->connect($dsnString, $awlData{user_awl_sql_username}, 
		$awlData{user_awl_sql_password}, { RaiseError => 0, PrintError => 0 });
	$awlDBH->{'mysql_auto_reconnect'} = 1;

    return ($awlDBH, $awlDBH);

  } else {
    die "Unknown database request: $type";
  };

};



#######################################################################
# CreateDBH subroutine
#######################################################################

# Create our database handles for both vpopmail and fdqueue
  sub CreateDBH {

    ($DBHread, $DBH) = dbhConnect('vpopmail');
    if (-e $queueMysqlConfigFile) {
	($QBHread, $QBH) = dbhConnect('fdqueue');
    } else {
	# No separate queue database, alias to main vpopmail one
	$QBH = $DBH;
	$QBHread = $DBHread;
    };
  };


##########################################################################
# 
# Subroutine to read a file in from the qmail/control directory
# returns undef if the file doesn't exist... defaults should be handled
# at the assignment level (e.g.:
#               my $pref = qmailControlFile('preflimit') || 25;
#   
##########################################################################

sub qmailControlFile {
    return unless $_[0] && open (FILE, '/etc/qmail/control/' . $_[0]);
    chomp (my $dat = <FILE>);
    $lastQmailFileMtime{$_[0]} = (stat _)[9];
    return $dat;
};

sub qmailListFile {
    return unless $_[0] && open (FILE, '/etc/qmail/control/' . $_[0]);
    my @returnList;
    push @returnList, $_ while (<FILE>);
    chomp @returnList;
    $lastQmailFileMtime{$_[0]} = (stat _)[9];
    return @returnList;
};

# Note, since a ref is being passed here, we short-circuit if the
# control file hasn't changed
sub qmailHashRefFile {
    return if $lastQmailFileMtime{$_[0]} && (stat '/etc/qmail/control/' . $_[0])[9] == $lastQmailFileMtime{$_[0]};
    return unless $_[0] && $_[1] && open (FILE, '/etc/qmail/control/' . $_[0]);

    %{$_[1]}=();	# clear existing contents
    (chomp && m/^[^#]/ && $_[1]->{$_}++) while (<FILE>);
    # my @meh = keys %{$_[1]};
    # print STDERR "qmailHashRef keys for $_[0]: " . scalar @meh . "\n";
    $lastQmailFileMtime{$_[0]} = (stat _)[9];
};


##########################################################################
# 
# Subroutine to read in our main list of virtual domains
# 
##########################################################################

sub ReadVirtDomains {

  # Clear out existing hash
    %domainResolve = ();
    %domainDirs = ();

    open (ASSIGN, $virtUserFile) or die "Can't open $virtUserFile: $!\n";
    while (<ASSIGN>) {

	next if m/^#/ || m/^\s*$/ || m/^\./;
	chomp;
	my ($domain, $domreal, $domdir) = (split(/:/, $_, 6))[0,1,4];

	# Do our regexs here to eliminate a potential cached symlink stat later
	$domdir =~ s#^.*/domains/#$fdConfig{'realDomainDirLoc'}/#o if $fdConfig{'realDomainDirLoc'};
	$domain =~ s/^\+?\!?(.*)-$/$1/;

	$domainResolve{$domain}=$domreal;
	$domainDirs{$domain}=$domdir if $domain eq $domreal;

    };
    $virtUserFileMtime = (stat ASSIGN)[9];
    close ASSIGN;

};


1;
