# Create the auto-whitelist and bayesian databases used by SpamAssassin
# 
# In order to enable this, your local.cf (or SpamAssassinConf file) must include:
#
#	user_awl_dsn			spamassassin
#	user_awl_sql_username		sauser
#	user_awl_sql_password		sapassword
#	user_awl_sql_table		awl
#	#user_awl_sql_override_username	NOT USED -- put something here if you want
#					all awl entries to be checked using a single
#					username (the 'username' field below), ie vpopmail
# 

CREATE DATABASE IF NOT EXISTS spamassassin;
USE spamassassin;

### AWL Table

CREATE TABLE IF NOT EXISTS awl (
  username VARCHAR(96) NOT NULL,
  email VARCHAR(127) NOT NULL,
  ip VARCHAR(15) NOT NULL,
  count INT UNSIGNED NOT NULL,
  totscore FLOAT NOT NULL,
  PRIMARY KEY (username, email, ip)
);


### Bayesian Statistics Tables

CREATE TABLE IF NOT EXISTS bayes_expire (
  id int(11) NOT NULL default '0',
  runtime int(11) NOT NULL default '0',
  KEY bayes_expire_idx1 (id)
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS bayes_global_vars (
  variable varchar(30) NOT NULL default '',
  value varchar(200) NOT NULL default '',
  PRIMARY KEY  (variable)
) TYPE=MyISAM;

INSERT INTO bayes_global_vars VALUES ('VERSION','3');

CREATE TABLE IF NOT EXISTS bayes_seen (
  id int(11) NOT NULL default '0',
  msgid varchar(200) binary NOT NULL default '',
  flag char(1) NOT NULL default '',
  PRIMARY KEY  (id,msgid)
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS bayes_token (
  id int(11) NOT NULL default '0',
  token char(5) NOT NULL default '',
  spam_count int(11) NOT NULL default '0',
  ham_count int(11) NOT NULL default '0',
  atime int(11) NOT NULL default '0',
  PRIMARY KEY  (id, token),
  INDEX bayes_token_idx1 (token),
  INDEX bayes_token_idx2 (id, atime)
) TYPE=MyISAM;

CREATE TABLE IF NOT EXISTS bayes_vars (
  id int(11) NOT NULL AUTO_INCREMENT,
  username varchar(200) NOT NULL default '',
  spam_count int(11) NOT NULL default '0',
  ham_count int(11) NOT NULL default '0',
  token_count int(11) NOT NULL default '0',
  last_expire int(11) NOT NULL default '0',
  last_atime_delta int(11) NOT NULL default '0',
  last_expire_reduce int(11) NOT NULL default '0',
  oldest_token_age int(11) NOT NULL default '2147483647',
  newest_token_age int(11) NOT NULL default '0',
  PRIMARY KEY  (id),
  UNIQUE bayes_vars_idx1 (username)
) TYPE=MyISAM;

